#ifndef _FILE_OUTPUT_H_
#define _FILE_OUTPUT_H_


/*
 * An object which writes simultaneously to multiple outputs. Applies the singleton
 * design pattern.
 * Output may consist of any of int,char,char*,float,double and Printable. Printable
 * is an interface which defines the pure virtual function 'toString()' which returns
 * the string representation of an object (similar to the convention found in
 * the java programming language).
 *
 * Author: Ziv Yaniv (zivy@cs.huji.ac.il).
 *
 *
 * Note: the documentation appears in the cpp file. This is to encourage any people
 *       who want to use this object to understand its inner workings. This is mostly
 *       an exercise in programming and an example of the Singleton design pattern.
 */


#include <stdio.h>
#ifdef WIN32
#include <vector>
#include <map>
#else
#include <vector.h>
#include <map.h>
#define _strdup strdup
#endif

using namespace std;

class Printable {
public:
	virtual char * toString() = 0;
};


class FileOutput {
public:
  const unsigned long STDOUT;
  const unsigned long STDERR;

  static FileOutput *getInstance();
  unsigned long addFile(char *name, const char *mode);
	char *fileName(unsigned long fileId);
  void closeFiles(unsigned long fileIds= 0xfffffffc);


  void print(unsigned long files,int i);
	void print(unsigned long files,char c);
  void print(unsigned long files,char *c);
  void print(unsigned long files,float f);
  void print(unsigned long files,double d);
  void print(unsigned long files,Printable &p);
  void println(unsigned long files,int i);
  void println(unsigned long files,char c);
  void println(unsigned long files,char *c);
  void println(unsigned long files,float f);
  void println(unsigned long files,double d);
	void println(unsigned long files,Printable &p);
	
protected:
  FileOutput();

  class FileData {
    public:
      FileData(char *name,unsigned long id) : _f(NULL), _id(id) {_name = _strdup(name);}
      ~FileData() {free(_name);}
      FILE *_f;
      char *_name;
      unsigned long _id;
  };

  class StrComparator {
  public:
    bool operator()(const char* str1, const char* str2) const {
      return(strcmp(str1, str2) < 0);
    }
  };

private:

	static FileOutput *_instance;
	map<char *, FileData *,StrComparator> _filesByName;
  map<unsigned long, FileData *> _filesById;
  unsigned long _freeFileIds;
};

#endif //_FILE_OUTPUT_H_
